@@ -1,7 +1,9 @@
 package SQL::Translator::Producer::POD;
 
 # -------------------------------------------------------------------
-# Copyright (C) 2002-2009 SQLFairy Authors
+# $Id: POD.pm,v 1.2 2003/06/10 03:49:35 kycl4rk Exp $
+# -------------------------------------------------------------------
+# Copyright (C) 2003 Ken Y. Clark <kclark@cpan.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -18,29 +20,9 @@ package SQL::Translator::Producer::POD;
 # 02111-1307  USA
 # -------------------------------------------------------------------
 
-=head1 NAME
-
-SQL::Translator::Producer::POD - POD producer for SQL::Translator
-
-=head1 SYNOPSIS
-
-  use SQL::Translator;
-
-  my $t = SQL::Translator->new( parser => '...', producer => 'POD', '...' );
-  print $t->translate;
-
-=head1 DESCRIPTION
-
-Creates a POD description of each table, field, index, and constraint.  
-A good starting point for text documentation of a schema.  You can 
-easily convert the output to HTML or text using "perldoc" or other 
-interesting formats using Pod::POM or Template::Toolkit's POD plugin.
-
-=cut
-
 use strict;
 use vars qw[ $VERSION ];
-$VERSION = '1.59';
+$VERSION = sprintf "%d.%02d", q$Revision: 1.2 $ =~ /(\d+)\.(\d+)/;
 
 use SQL::Translator::Schema::Constants;
 use SQL::Translator::Utils qw(header_comment);
@@ -51,9 +33,8 @@ sub produce {
     my $schema      = $t->schema;
     my $schema_name = $schema->name || 'Schema';
     my $args        = $t->producer_args;
-    my $title       = $args->{'title'} || $schema_name;
 
-    my $pod = "=pod\n\n=head1 DESCRIPTION\n\n$title\n\n=head1 TABLES\n\n";
+    my $pod = "=pod\n\n=head1 DESCRIPTION\n\n$schema_name\n\n=head1 TABLES\n\n";
 
     for my $table ( $schema->get_tables ) {
         my $table_name = $table->name or next;
@@ -114,11 +95,11 @@ sub produce {
                 }
 
                 if ( my $update = $c->on_update ) {
-                    $pod .= "=item * On update = $update\n\n";
+                    $pod .= "=item * On update = $update";
                 }
 
                 if ( my $delete = $c->on_delete ) {
-                    $pod .= "=item * On delete = $delete\n\n";
+                    $pod .= "=item * On delete = $delete";
                 }
 
                 $pod .= "=back\n\n";
@@ -140,18 +121,27 @@ sub produce {
 # William Blake
 # -------------------------------------------------------------------
 
-=pod
+=head1 NAME
+
+SQL::Translator::Producer::POD - POD producer for SQL::Translator
+
+=head1 SYNOPSIS
+
+  use SQL::Translator::Producer::POD;
+
+=head1 DESCRIPTION
+
+Creates a POD description of each table, field, index, and constraint.  
+A good starting point for text documentation of a schema.  You can 
+easily convert the output to HTML or text using "perldoc" or other 
+interesting formats using Pod::POM or Template::Toolkit.
 
 =head1 AUTHOR
 
-Ken Y. Clark E<lt>kclark@cpan.orgE<gt>.
-
-=head2 CONTRIBUTORS
-
-Jonathan Yu E<lt>frequency@cpan.orgE<gt>
+Ken Y. Clark E<lt>kclark@cpan.orgE<gt>
 
 =head1 SEE ALSO
 
-perldoc, perlpod, Pod::POM, Template::Manual::Plugins.
+perldoc, perlpod, Pod::POM, Template::Toolkit.
 
 =cut
