@@ -1,7 +1,9 @@
 package SQL::Translator::Schema::Constants;
 
 # ----------------------------------------------------------------------
-# Copyright (C) 2002-2009 SQLFairy Authors
+# $Id: Constants.pm,v 1.4 2003/06/06 22:34:00 kycl4rk Exp $
+# ----------------------------------------------------------------------
+# Copyright (C) 2003 Ken Y. Clark <kclark@cpan.org>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -20,11 +22,11 @@ package SQL::Translator::Schema::Constants;
 
 =head1 NAME
 
-SQL::Translator::Schema::Constants - constants module
+SQL::Translator::CMap::Constants - constants module
 
 =head1 SYNOPSIS
 
-  use SQL::Translator::Schema::Constants;
+  use SQL::Translator::CMap::Constants;
 
   $table->add_constraint(
       name => 'foo',
@@ -33,27 +35,7 @@ SQL::Translator::Schema::Constants - constants module
 
 =head1 DESCRIPTION
 
-This module exports the following constants for Schema features;
-
-=over 4
-
-=item CHECK_C
-
-=item FOREIGN_KEY
-
-=item FULL_TEXT
-
-=item NOT_NULL
-
-=item NORMAL
-
-=item NULL
-
-=item PRIMARY_KEY
-
-=item UNIQUE
-
-=back
+This module exports a several constants to like "primary key," etc. 
 
 =cut
 
@@ -61,13 +43,12 @@ use strict;
 use base qw( Exporter );
 use vars qw( @EXPORT $VERSION );
 require Exporter;
-$VERSION = '1.59';
+$VERSION = (qw$Revision: 1.4 $)[-1];
 
 @EXPORT = qw[ 
     CHECK_C
     FOREIGN_KEY
     FULL_TEXT
-    SPATIAL
     NOT_NULL
     NORMAL
     NULL
@@ -84,8 +65,6 @@ use constant FOREIGN_KEY => 'FOREIGN KEY';
 
 use constant FULL_TEXT => 'FULLTEXT';
 
-use constant SPATIAL => 'SPATIAL';
-
 use constant NOT_NULL => 'NOT NULL';
 
 use constant NORMAL => 'NORMAL';
@@ -98,12 +77,17 @@ use constant UNIQUE => 'UNIQUE';
 
 1;
 
-# ----------------------------------------------------------------------
-
 =pod
 
 =head1 AUTHOR
 
-Ken Y. Clark E<lt>kclark@cpan.orgE<gt>.
+Ken Y. Clark E<lt>kclark@cpan.orgE<gt>
+
+=head1 COPYRIGHT
+
+Copyright (c) 2003
+
+This library is free software;  you can redistribute it and/or modify 
+it under the same terms as Perl itself.
 
 =cut
